@@ -2,7 +2,7 @@ import json
 import re
 import sys
 from collections import defaultdict
-from datetime import date, datetime
+from datetime import date, datetime, timezone
 from functools import wraps
 from typing import Any, Callable, DefaultDict, List, Optional, Sequence, TextIO, Union
 
@@ -10,9 +10,12 @@ import click
 import requests
 
 from . import __version__
-from .cve_api import CveApi, CveRecordValidationError
+from .cve_api import CveApi, CveRecord, CveRecordValidationError
 
-CVE_RE = re.compile(r"^CVE-[12]\d{3}-\d{4,}$")
+# Check CVE IDs to use valid years (first one assigned was in 1999; hopefully we'll transcend
+# software vulnerabilities in year 3000+), and disallow ID of 0000 and leading zeros for 4+ IDs
+# (e.g. -00001 is an invalid ID).
+CVE_RE = re.compile(r"CVE-(?:1999|2\d{3})-(?!0{4})(?:0\d{3}|[1-9]\d{3,})")
 CONTEXT_SETTINGS = {
     "help_option_names": ["-h", "--help"],
     "max_content_width": 100,
@@ -39,7 +42,32 @@ def validate_year(
 
 
 def human_ts(ts: str) -> str:
-    return datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S.%fZ").strftime("%c")
+    """Return a consistent, human-readable datetime.
+
+    If the format cannot be parsed against RFC3339 or ISO8601, return the original raw value.
+
+    Note: this helper function should only be used for printing datetime values.
+    """
+    dt = None
+    for dt_format in (
+        "%Y-%m-%dT%H:%M:%S",  # 2019-03-27T19:20:26
+        "%Y-%m-%dT%H:%M:%S.%f%z",  # 2019-03-27T19:20:26.123+01:00
+        "%Y-%m-%dT%H:%M:%S%z",  # 2019-03-27T19:20:26+01:00
+        "%Y-%m-%dT%H:%M:%S.%fZ",  # 2019-03-27T19:20:26.123Z
+        "%Y-%m-%dT%H:%M:%S%Z",  # 2019-03-27T19:20:26Z
+    ):
+        try:
+            dt = datetime.strptime(ts, dt_format)
+        except ValueError:
+            pass
+        else:
+            break
+    if dt:
+        if not dt.tzinfo:
+            dt = dt.replace(tzinfo=timezone.utc)
+        return dt.strftime("%c %z")
+    else:
+        return ts
 
 
 def print_cve_id(cve: dict) -> None:
@@ -48,17 +76,29 @@ def print_cve_id(cve: dict) -> None:
     # CVEs reserved by other CNAs do not include information on who requested them and when.
     if "requested_by" in cve:
         click.echo(f"├─ Owning CNA:\t{cve['owning_cna']}")
-        click.echo(f"├─ Reserved by:\t{cve['requested_by']['user']} ({cve['requested_by']['cna']})")
-        click.echo(f"└─ Reserved on:\t{human_ts(cve['reserved'])}")
+        click.echo(
+            "├─ Reserved by:\t"
+            f"{cve['requested_by'].get('user', 'n/a')} ({cve['requested_by']['cna']})"
+        )
+        if "time" in cve:
+            click.echo(f"├─ Reserved on:\t{human_ts(cve['reserved'])}")
+            click.echo(f"└─ Updated on:\t{human_ts(cve['time']['modified'])}")
+        else:
+            click.echo(f"└─ Reserved on:\t{human_ts(cve['reserved'])}")
     else:
-        click.echo(f"└─ Owning CNA:\t{cve['owning_cna']}")
+        if "dateUpdated" in cve:
+            click.echo(f"├─ Owning CNA:\t{cve['owning_cna']}")
+            click.echo(f"└─ Updated on:\t{human_ts(cve['dateUpdated'])}")
+        else:
+            click.echo(f"└─ Owning CNA:\t{cve['owning_cna']}")
 
 
 def print_cve_record(cve: dict) -> None:
     click.secho(cve["cveMetadata"]["cveId"], bold=True)
     click.echo(f"├─ State:\t{cve['cveMetadata']['state']}")
     click.echo(f"├─ Owning CNA:\t{cve['cveMetadata']['assignerShortName']}")
-    click.echo(f"└─ Reserved on:\t{human_ts(cve['cveMetadata']['dateReserved'])}")
+    click.echo(f"├─ Reserved on:\t{human_ts(cve['cveMetadata'].get('dateReserved', 'N/A'))}")
+    click.echo(f"└─ Updated on:\t{human_ts(cve['cveMetadata'].get('dateUpdated', 'N/A'))}")
 
 
 def print_table(lines: Sequence[Sequence[str]], highlight_header: bool = True) -> None:
@@ -126,24 +166,28 @@ def handle_cve_api_error(func: Callable) -> Callable:
             return func(*args, **kwargs)
         except requests.exceptions.RequestException as exc:
             error = str(exc)
-            details = None
-            if getattr(exc, "response", None) is not None:
+            details = ""
+            response = getattr(exc, "response", None)
+            if response is not None:
                 try:
-                    details = exc.response.json()
+                    details = str(response.json())
                 except ValueError:
-                    details = exc.response.content
+                    details = response.content.decode("utf-8")
             print_error(error, details)
         except CveRecordValidationError as exc:
             click.secho("ERROR: ", bold=True, nl=False)
             click.echo("CVE record is not valid against the v5 JSON schema:")
             for error in exc.errors:
                 click.echo(f"  {error}")
+        cmd_ctx = next((arg for arg in args if isinstance(arg, click.Context)), None)
+        cmd_name = cmd_ctx.command.name if cmd_ctx else "<command>"
+        click.echo(f"\nUse `cve {cmd_name} -h` for more information on how to use this command")
         sys.exit(1)
 
     return wrapped
 
 
-def print_error(msg: str, details: Optional[str]) -> None:
+def print_error(msg: str, details: str) -> None:
     click.secho("ERROR: ", bold=True, nl=False)
     click.echo(msg)
     if details:
@@ -179,9 +223,38 @@ class Config:
         )
 
 
-@click.group(context_settings=CONTEXT_SETTINGS)
+class SkipRequiredIf(click.Group):
+    def parse_args(self, ctx: click.Context, args: list) -> list:
+        """Mark global required options as not required in certain cases
+
+        If any help options are used, we can skip directly to showing the help without requiring
+        users to specify authentication values.
+
+        If certain subcommands are used that don't require authentication (e.g. `ping`), do not
+        require authentication values.
+        """
+        if any(arg in ctx.help_option_names for arg in args) or any(
+            arg in ("ping", "show", "validate") for arg in args
+        ):
+            # Iterate over all options and flip them to not required and not to prompt for input.
+            for param in self.params:
+                if isinstance(param, click.Option):
+                    param.required = False
+                    # Type ignored due to `"Option" has no attribute "prompt_required"` error:
+                    # https://github.com/pallets/click/blob/d0af32d8/src/click/core.py#L2455
+                    param.prompt_required = False  # type: ignore
+        return super(SkipRequiredIf, self).parse_args(ctx, args)
+
+
+@click.group(context_settings=CONTEXT_SETTINGS, cls=SkipRequiredIf)
 @click.option(
-    "-u", "--username", envvar="CVE_USER", required=True, help="Your username (env var: CVE_USER)"
+    "-u",
+    "--username",
+    envvar="CVE_USER",
+    required=True,
+    help="Your username (env var: CVE_USER)",
+    prompt="Username",
+    hide_input=False,
 )
 @click.option(
     "-o",
@@ -189,6 +262,8 @@ class Config:
     envvar="CVE_ORG",
     required=True,
     help="Your CNA organization short name (env var: CVE_ORG)",
+    prompt="Organization",
+    hide_input=False,
 )
 @click.option(
     "-a",
@@ -262,29 +337,31 @@ def publish(
     cve_json_file: Optional[TextIO],
     print_raw: bool,
 ) -> None:
-    """Publish a CVE record for a reserved (or rejected) CVE ID.
+    """Publish a CNA container of a CVE record for a reserved (or rejected) CVE ID.
 
-    If the CVE is already published, this action will update its record. A published CVE can only be
-    moved to the rejected state with an appropriate reject record (see `cve reject`). A published
-    CVE cannot be moved back to the reserved state.
+    If the CVE is already published, this action will update the CNA container in its record. A
+    published CVE can only be moved to the rejected state with an appropriate reject record (see
+    `cve reject`). A published CVE cannot be moved back to the reserved state.
 
-    The CVE record can be specified as a string:
+    The CNA container can be specified as a string:
 
       cve publish CVE-2022-1234 -j '{"affected": [], "descriptions": [], "references": {}, ...}'
 
     Or passed in a file:
 
-      cve publish CVE-2022-1234 -f v5_record.json
+      cve publish CVE-2022-1234 -f cve.json
 
-    For information on the required properties in a given CVE JSON record, see the schema in:\n
-    https://github.com/CVEProject/cve-schema/blob/master/schema/v5.0/CVE_JSON_5.0_schema.json
+    For information on the required properties in a given CNA container of a CVE record,
+    see the schema in:\n
+    https://cveproject.github.io/cve-schema/schema/docs/
 
-    Because the CVE Services API only expects the cnaPublishedContainer contents of the full record,
-    the record you pass to this command can specify just that data, and not the full record.
+    Because the CVE Services API only expects the CNA container contents of the full CVE v5 record,
+    the data you pass to this command can specify just the attributes defined by the
+    cnaPublishedContainer or cnaRejectedContainer subschemas, and not the full schema record.
     """
     if cve_json_file is not None and cve_json_str is not None:
         raise click.BadParameter(
-            "cannot use both `-f/--cve-json-file` and `-j/--cve-json` to provide a CVE JSON record."
+            "cannot use both `-f/--cve-json-file` and `-j/--cve-json` to provide a CNA JSON data."
         )
 
     try:
@@ -294,19 +371,18 @@ def publish(
             cve_json = json.load(cve_json_file)
         else:
             raise click.BadParameter(
-                "must provide CVE JSON record using one of: "
-                "`-f/--cve-json-file` or `-j/--cve-json`."
+                "must provide CNA JSON data using one of: `-f/--cve-json-file` or `-j/--cve-json`."
             )
     except json.JSONDecodeError as exc:
-        print_error(msg="CVE data is not valid JSON", details=str(exc))
+        print_error(msg="CNA data is not valid JSON", details=str(exc))
         return
 
     if ctx.obj.interactive:
-        click.echo("You are about to publish a CVE record for ", nl=False)
+        click.echo("You are about to publish a CNA container in the CVE record for ", nl=False)
         click.secho(cve_id, bold=True, nl=False)
         click.echo(" using the following input:\n\n", nl=False)
         print_json_data(cve_json)
-        if not click.confirm("\n\nDo you want to continue?"):
+        if not click.confirm("\nDo you want to continue?"):
             click.echo("Exiting...")
             sys.exit(0)
         click.echo()
@@ -326,6 +402,94 @@ def publish(
     else:
         click.echo("Published the following CVE:\n")
         print_cve_record(response_data["created"] if created else response_data["updated"])
+        click.echo(f'\nAPI response: {response_data["message"]}')
+
+
+@cli.command()
+@click.argument("cve_id", callback=validate_cve)
+@click.option(
+    "-j",
+    "--adp-json",
+    "adp_json_str",
+    type=click.STRING,
+    help="JSON body of the ADP container to publish.",
+)
+@click.option(
+    "-f",
+    "--adp-json-file",
+    type=click.File(),
+    help="File containing JSON body of ADP container to publish.",
+)
+@click.option("--raw", "print_raw", default=False, is_flag=True, help="Print response JSON.")
+@click.pass_context
+@handle_cve_api_error
+def publish_adp(
+    ctx: click.Context,
+    cve_id: str,
+    adp_json_str: Optional[str],
+    adp_json_file: Optional[TextIO],
+    print_raw: bool,
+) -> None:
+    """Add or update an ADP container in a CVE record for a published CVE ID.
+
+    NOTE: a published ADP container cannot be removed, only updated with new data.
+
+    The ADP container can be specified as a string:
+
+      cve publish-adp CVE-2022-1234 -j '{"affected": [], "descriptions": [], "references": {}, ...}'
+
+    Or passed in a file:
+
+      cve publish-adp CVE-2022-1234 -f adp.json
+
+    For information on the required properties in a given ADP container of a CVE record,
+    see the schema in:\n
+    https://cveproject.github.io/cve-schema/schema/docs/
+
+    Because the CVE Services API only expects the ADP container contents of the full CVE v5 record,
+    the data you pass to this command can specify just the attributes defined by the adpContainer
+    subschema.
+    """
+    if adp_json_file is not None and adp_json_str is not None:
+        raise click.BadParameter(
+            "cannot use both `-f/--adp-json-file` and `-j/--adp-json` to provide ADP JSON data."
+        )
+    try:
+        if adp_json_str is not None:
+            cve_json = json.loads(adp_json_str)
+        elif adp_json_file is not None:
+            cve_json = json.load(adp_json_file)
+        else:
+            raise click.BadParameter(
+                "must provide ADP JSON record using one of: "
+                "`-f/--adp-json-file` or `-j/--adp-json`."
+            )
+    except json.JSONDecodeError as exc:
+        print_error(msg="ADP data is not valid JSON", details=str(exc))
+        return
+
+    if ctx.obj.interactive:
+        click.echo("You are about to publish an ADP record for ", nl=False)
+        click.secho(cve_id, bold=True, nl=False)
+        click.echo(" using the following input:\n\n", nl=False)
+        print_json_data(cve_json)
+        if not click.confirm(
+            "\nADP containers cannot be removed once published; they can only be updated.\n"
+            "Do you want to continue?"
+        ):
+            click.echo("Exiting...")
+            sys.exit(0)
+        click.echo()
+
+    cve_api = ctx.obj.cve_api
+    response_data = cve_api.publish_adp(cve_id, cve_json)
+
+    if print_raw:
+        print_json_data(response_data)
+    else:
+        click.echo("Published an ADP container for:\n")
+        print_cve_record(response_data["updated"])
+        click.echo(f'\nAPI response: {response_data["message"]}')
 
 
 @cli.command()
@@ -360,6 +524,8 @@ def reject(
     A rejected CVE without a record can be moved to the reserved state. A published CVE can only
     be rejected with an accompanying record. Reserved CVEs can be rejected with or without a record.
 
+    Note: Rejecting a published CVE removes both the CNA and ADP data of that record.
+
     The CVE reject record can be specified as a string:
 
       cve reject CVE-2022-1234 -j '{"rejectedReasons": [{"lang": "en", "value": "A reason."}]}'
@@ -369,7 +535,7 @@ def reject(
       cve reject CVE-2022-1234 -f v5_reject_record.json
 
     For information on the required properties in a given CVE JSON record, see the schema in:\n
-    https://github.com/CVEProject/cve-schema/blob/master/schema/v5.0/CVE_JSON_5.0_schema.json
+    https://cveproject.github.io/cve-schema/schema/docs/
 
     Because the CVE Services API only expects the cnaRejectedContainer contents of the full record,
     the record you pass to this command can specify just that data, and not the full record.
@@ -431,6 +597,7 @@ def reject(
     else:
         click.echo("Rejected the following CVE:\n")
         print_cve_record(response_data["created"] if created else response_data["updated"])
+        click.echo(f'\nAPI response: {response_data["message"]}')
 
 
 @cli.command()
@@ -534,19 +701,49 @@ def reserve(ctx: click.Context, random: bool, year: str, count: int, print_raw:
     "--show-record",
     default=False,
     is_flag=True,
-    help="Show full CVE record in JSON v5 format.",
+    help="Show full CVE v5 record.",
+)
+@click.option(
+    "-c",
+    "--show-cna",
+    default=False,
+    is_flag=True,
+    help="Show only the CNA container of a CVE v5 record.",
+)
+@click.option(
+    "-a",
+    "--show-adp",
+    multiple=True,
+    is_flag=False,
+    flag_value="",
+    help=(
+        "Show all ADP containers, or a specific one identified by the owning org (option can "
+        "be used multiple times)."
+    ),
+    show_default="all ADP containers",
 )
 @click.option("--raw", "print_raw", default=False, is_flag=True, help="Print response JSON.")
 @click.argument("cve_id", callback=validate_cve)
 @click.pass_context
 @handle_cve_api_error
-def show_cve(ctx: click.Context, show_record: bool, print_raw: bool, cve_id: str) -> None:
-    """Display a specific CVE ID (and optionally its record) owned by your CNA."""
+def show_cve(
+    ctx: click.Context,
+    show_record: bool,
+    show_cna: bool,
+    show_adp: str,
+    print_raw: bool,
+    cve_id: str,
+) -> None:
+    """Display a specific CVE ID (and optionally its record)."""
+    if [show_record, show_cna, bool(show_adp)].count(True) > 1:
+        raise click.BadParameter(
+            "use one of: `-r/--show-record` OR `-c/--show-cna` OR `-a/--show-adp`."
+        )
     cve_api = ctx.obj.cve_api
 
     cve_id_data = cve_api.show_cve_id(cve_id=cve_id)
     cve_record_data = {}
-    if show_record:
+    if show_record or show_cna or show_adp:
         try:
             cve_record_data = cve_api.show_cve_record(cve_id=cve_id)
         except requests.exceptions.HTTPError as exc:
@@ -554,23 +751,63 @@ def show_cve(ctx: click.Context, show_record: bool, print_raw: bool, cve_id: str
             if exc.response.status_code != 404 or error_msg != cve_api.Errors.RECORD_DOES_NOT_EXIST:
                 raise exc
 
+    org_to_adp = {}
+    # Convert tuple of specified org names to a set; if no orgs were specified but the `-a`
+    # option was used, the tuple will use the default empty string as the only value in the
+    # tuple (i.e. `('',)`). A resulting empty set indicates we should print all containers.
+    orgs = sorted({org for org in show_adp if org})
+    if show_adp and "adp" in cve_record_data.get("containers", {}):
+        for adp_container in cve_record_data["containers"]["adp"]:
+            org = adp_container["providerMetadata"]["shortName"]
+            if orgs and org not in orgs:
+                continue
+            org_to_adp[org] = adp_container
+
     if print_raw:
-        # Display CVE record data only if we're showing the record. Otherwise, show the CVE ID
-        # data only.
+        # Display CVE record data (or parts of it) only. Otherwise, show the CVE ID data only.
         if show_record:
             print_json_data(cve_record_data)
+        elif show_cna:
+            cna_container = (
+                cve_record_data["containers"]["cna"] if cve_record_data else cve_record_data
+            )
+            print_json_data(cna_container)
+        elif show_adp:
+            print_json_data(list(org_to_adp.values()))
         else:
             print_json_data(cve_id_data)
     else:
         print_cve_id(cve_id_data)
-        # If we're showing the CVE record, display it as either the raw JSON if it exists or show
-        # an informational message if it does not.
+        # If we're showing the CVE record (or any of its parts), display it as either the raw
+        # JSON if it exists or show an informational message if it does not.
         if show_record:
             click.secho("-----", bold=True)
             if cve_record_data:
                 print_json_data(cve_record_data)
             else:
                 click.echo("CVE record does not exist.")
+        elif show_cna:
+            click.secho("-----", bold=True)
+            if cve_record_data:
+                print_json_data(cve_record_data["containers"]["cna"])
+            else:
+                click.echo("CVE record does not exist.")
+        elif show_adp:
+            click.secho("-----", bold=True)
+            if org_to_adp:
+                count = len(org_to_adp)
+                for idx, (org, data) in enumerate(org_to_adp.items()):
+                    click.secho(f"{org} ADP data:", bold=True)
+                    print_json_data(data)
+                    # Add empty line between ADP containers except after the last one.
+                    if idx + 1 < count:
+                        click.echo()
+            else:
+                if orgs:
+                    orgs_text = ", ".join(orgs)
+                    click.echo(f"CVE record does not contain ADP data from org(s): {orgs_text}.")
+                else:
+                    click.echo("CVE record does not contain any ADP data.")
 
 
 @cli.command(name="list")
@@ -615,7 +852,7 @@ def list_cves(
     if sort_by:
         key = sort_by.lower()
         if key == "user":
-            cves.sort(key=lambda x: x["requested_by"]["user"])
+            cves.sort(key=lambda x: x["requested_by"].get("user", ""))
         elif key == "cve_id":
             cves.sort(key=lambda x: natural_cve_sort(x["cve_id"]))
         elif key == "reserved_ts":
@@ -633,7 +870,7 @@ def list_cves(
                 cve["cve_id"],
                 cve["state"],
                 cve["owning_cna"],
-                f"{cve['requested_by']['user']} ({cve['requested_by']['cna']})",
+                f"{cve['requested_by'].get('user') or 'n/a'} ({cve['requested_by']['cna']})",
                 human_ts(cve["reserved"]),
             )
         )
@@ -671,7 +908,7 @@ def quota(ctx: click.Context, print_raw: bool) -> None:
     "-u",
     "--username",
     help="Specify the user to show.",
-    show_default="Current user specified in -u/--username/CVE_USER",
+    show_default="Current user specified in top-level -u/--username/$CVE_USER",
 )
 @click.option("--raw", "print_raw", default=False, is_flag=True, help="Print response JSON.")
 @click.pass_context
@@ -696,8 +933,8 @@ def show_user(ctx: click.Context, username: Optional[str], print_raw: bool) -> N
 @click.option(
     "-u",
     "--username",
+    required=True,
     help="User whose API key should be reset (only ADMIN role users can update other users).",
-    show_default="Current user specified in global -u/--username/CVE_USER",
 )
 @click.option("--raw", "print_raw", default=False, is_flag=True, help="Print response JSON.")
 @click.pass_context
@@ -705,6 +942,8 @@ def show_user(ctx: click.Context, username: Optional[str], print_raw: bool) -> N
 def reset_key(ctx: click.Context, username: Optional[str], print_raw: bool) -> None:
     """Reset a user's personal access token (API key).
 
+    You must explicitly specify the user being updated using the `-u` option.
+
     This API key is used to authenticate each request to the CVE API.
     """
     cve_api = ctx.obj.cve_api
@@ -727,8 +966,8 @@ def reset_key(ctx: click.Context, username: Optional[str], print_raw: bool) -> N
 @click.option(
     "-u",
     "--username",
+    required=True,
     help="Username of the user being updated (only ADMIN role users can update other users).",
-    show_default="Current user specified in global -u/--username/CVE_USER",
 )
 @click.option(
     "--mark-active/--mark-inactive", "active", default=None, help="Mark user as active or inactive."
@@ -744,6 +983,8 @@ def reset_key(ctx: click.Context, username: Optional[str], print_raw: bool) -> N
 def update_user(ctx: click.Context, username: Optional[str], **opts_data: dict) -> None:
     """Update a user.
 
+    You must explicitly specify the user being updated using the `-u` option.
+
     To reset a user's API key, use `cve user reset-key`.
     """
     print_raw = opts_data.pop("print_raw")
@@ -898,6 +1139,71 @@ def users(ctx: click.Context, print_raw: bool, no_header: bool) -> None:
     print_table(lines, highlight_header=not no_header)
 
 
+@cli.command(
+    help=f"Validate a CVE record against the "
+    f"{CveRecord.Schemas.V5_SCHEMA.rsplit('_', maxsplit=1)[1].removesuffix('.json')} "
+    f"CVE JSON (sub)schema."
+)
+@click.option(
+    "-j",
+    "--cve-json",
+    "cve_json_str",
+    type=click.STRING,
+    help="JSON body of CVE record.",
+)
+@click.option(
+    "-f",
+    "--cve-json-file",
+    type=click.File(),
+    help="File containing JSON body of a CVE record.",
+)
+@click.option(
+    "-s",
+    "--schema-type",
+    default="cna-published",
+    show_default=True,
+    type=click.Choice(["full", "cna-published", "cna-rejected", "adp"], case_sensitive=False),
+    help="Specific type of schema to validate against",
+)
+def validate(
+    cve_json_str: Optional[str], cve_json_file: Optional[TextIO], schema_type: Optional[str]
+) -> None:
+    if cve_json_file is not None and cve_json_str is not None:
+        raise click.BadParameter(
+            "cannot use both `-f/--cve-json-file` and `-j/--cve-json` to provide a CVE JSON."
+        )
+
+    try:
+        if cve_json_str is not None:
+            cve_json = json.loads(cve_json_str)
+        elif cve_json_file is not None:
+            cve_json = json.load(cve_json_file)
+        else:
+            raise click.BadParameter(
+                "must provide CVE JSON data using one of: `-f/--cve-json-file` or `-j/--cve-json`."
+            )
+    except json.JSONDecodeError as exc:
+        print_error(msg="CVE data is not valid JSON", details=str(exc))
+        return
+
+    if schema_type == "full":
+        schema_path = CveRecord.Schemas.V5_SCHEMA
+    elif schema_type == "cna-published":
+        schema_path = CveRecord.Schemas.CNA_PUBLISHED
+    elif schema_type == "cna-rejected":
+        schema_path = CveRecord.Schemas.CNA_REJECTED
+    else:
+        schema_path = CveRecord.Schemas.ADP
+
+    try:
+        CveRecord.validate(cve_json, schema_path=schema_path)
+    except CveRecordValidationError as exc:
+        click.echo(exc)
+        sys.exit(1)
+    else:
+        click.echo("CVE record is valid!")
+
+
 @cli.command()
 @click.pass_context
 @handle_cve_api_error
