@@ -1,4 +1,6 @@
 import json
+import re
+from pathlib import Path
 from unittest import mock
 
 from click.testing import CliRunner
@@ -29,7 +31,48 @@ def test_cve_id_show(show_cve_record, show_cve_id):
         "├─ State:\tRESERVED\n"
         "├─ Owning CNA:\tacme\n"
         "├─ Reserved by:\tjack@example.com (acme)\n"
-        "└─ Reserved on:\tThu Jan 14 18:35:17 2021\n"
+        "├─ Reserved on:\tThu Jan 14 18:35:17 2021 +0000\n"
+        "└─ Updated on:\tThu Jan 14 18:35:17 2021 +0000\n"
+    )
+    assert not show_cve_record.called
+
+
+@mock.patch("cvelib.cli.CveApi.show_cve_id")
+@mock.patch("cvelib.cli.CveApi.show_cve_record")
+def test_reserved_not_owned_cve_id(show_cve_record, show_cve_id):
+    show_cve_id.return_value = {
+        "cve_id": "CVE-2099-0001",
+        "cve_year": "2099",
+        "owning_cna": "[REDACTED]",
+        "state": "RESERVED",
+    }
+
+    runner = CliRunner()
+    result = runner.invoke(cli, DEFAULT_OPTS + ["show", "CVE-2099-0001"])
+    assert result.exit_code == 0, result.output
+    assert result.output == "CVE-2099-0001\n├─ State:\tRESERVED\n└─ Owning CNA:\t[REDACTED]\n"
+    assert not show_cve_record.called
+
+
+@mock.patch("cvelib.cli.CveApi.show_cve_id")
+@mock.patch("cvelib.cli.CveApi.show_cve_record")
+def test_published_not_owned_cve_id(show_cve_record, show_cve_id):
+    show_cve_id.return_value = {
+        "cve_id": "CVE-2099-0001",
+        "cve_year": "2099",
+        "owning_cna": "acme",
+        "dateUpdated": "2000-02-08T17:22:07.669Z",
+        "state": "PUBLISHED",
+    }
+
+    runner = CliRunner()
+    result = runner.invoke(cli, DEFAULT_OPTS + ["show", "CVE-2099-0001"])
+    assert result.exit_code == 0, result.output
+    assert result.output == (
+        "CVE-2099-0001\n"
+        "├─ State:\tPUBLISHED\n"
+        "├─ Owning CNA:\tacme\n"
+        "└─ Updated on:\tTue Feb  8 17:22:07 2000 +0000\n"
     )
     assert not show_cve_record.called
 
@@ -79,7 +122,8 @@ def test_cve_show_full(show_cve_record, show_cve_id):
         "├─ State:\tRESERVED\n"
         "├─ Owning CNA:\tacme\n"
         "├─ Reserved by:\tjack@example.com (acme)\n"
-        "└─ Reserved on:\tThu Jan 14 18:35:17 2021\n"
+        "├─ Reserved on:\tThu Jan 14 18:35:17 2021 +0000\n"
+        "└─ Updated on:\tThu Jan 14 18:35:17 2021 +0000\n"
     )
     # Don't bother checking the data since we provide it as a fixture anyway. Simply check that a
     # JSON is displayed and the length of the printed string is something reasonable.
@@ -87,6 +131,81 @@ def test_cve_show_full(show_cve_record, show_cve_id):
     assert len(printed_cve_record) > 100
 
 
+@mock.patch("cvelib.cli.CveApi.show_cve_id")
+@mock.patch("cvelib.cli.CveApi.show_cve_record")
+def test_cve_show_adp(show_cve_record, show_cve_id):
+    show_cve_id.return_value = {
+        "cve_id": "CVE-2099-1000",
+        "cve_year": "2099",
+        "owning_cna": "acme",
+        "requested_by": {"cna": "acme", "user": "jack@example.com"},
+        "reserved": "2021-01-14T18:35:17.928Z",
+        "state": "PUBLISHED",
+        "time": {"created": "2021-01-14T18:35:17.469Z", "modified": "2021-01-14T18:35:17.929Z"},
+    }
+    show_cve_record.return_value = {
+        "containers": {
+            "cna": {
+                "providerMetadata": {
+                    "dateUpdated": "2022-09-27T15:29:12.964Z",
+                    "orgId": "65fe0718-9a55-4e29-8e61-d4ddf6d83e28",
+                    "shortName": "acme",
+                },
+                "references": [{"url": "https://example.com"}],
+            },
+            "adp": [
+                {
+                    "providerMetadata": {
+                        "dateUpdated": "2022-09-27T15:29:12.964Z",
+                        "orgId": "12345678-9a55-4e29-8e61-d4ddf6d83e28",
+                        "shortName": "alice-corp",
+                    },
+                    "references": [{"url": "https://example.com/alice"}],
+                },
+                {
+                    "providerMetadata": {
+                        "dateUpdated": "2022-09-27T15:29:12.964Z",
+                        "orgId": "87654321-9a55-4e29-8e61-d4ddf6d83e28",
+                        "shortName": "bob-corp",
+                    },
+                    "references": [{"url": "https://example.com/bob"}],
+                },
+            ],
+        },
+        "cveMetadata": {
+            "assignerOrgId": "65fe0718-9a55-4e29-8e61-d4ddf6d83e28",
+            "assignerShortName": "acme",
+            "cveId": "CVE-2099-1000",
+            "dateRejected": "2022-09-27T15:26:42.117Z",
+            "dateReserved": "2021-01-14T18:35:17.469Z",
+            "dateUpdated": "2022-09-27T15:29:12.964Z",
+            "state": "PUBLISHED",
+        },
+        "dataType": "CVE_RECORD",
+        "dataVersion": "5.0",
+    }
+
+    runner = CliRunner()
+    result = runner.invoke(cli, DEFAULT_OPTS + ["show", "CVE-2099-1000", "--show-adp"])
+    assert result.exit_code == 0, result.output
+    _, _, printed_cve_record = result.output.partition("-----")
+    # Don't bother checking the data since we provide it as a fixture anyway. Simply check that both
+    # ADP containers are present.
+    assert "alice-corp ADP data:" in printed_cve_record
+    assert "bob-corp ADP data:" in printed_cve_record
+
+    result = runner.invoke(cli, DEFAULT_OPTS + ["show", "CVE-2099-1000", "--show-adp", "bob-corp"])
+    assert result.exit_code == 0, result.output
+    _, _, printed_cve_record = result.output.partition("-----")
+    assert "alice-corp ADP data:" not in printed_cve_record
+    assert "bob-corp ADP data:" in printed_cve_record
+
+    result = runner.invoke(cli, DEFAULT_OPTS + ["show", "CVE-2099-1000", "--show-adp", "random"])
+    assert result.exit_code == 0, result.output
+    _, _, printed_cve_record = result.output.partition("-----")
+    assert printed_cve_record.strip() == "CVE record does not contain ADP data from org(s): random."
+
+
 def test_cve_list():
     cves = [
         {
@@ -133,16 +252,19 @@ def test_cve_list():
         assert result.exit_code == 0, result.output
         assert result.output == (
             "CVE ID          STATE       OWNING CNA   RESERVED BY   RESERVED ON\n"
-            "CVE-2021-3001   RESERVED    acme         bob (acme)    Thu Jan 14 18:32:19 2021\n"
-            "CVE-2021-3002   PUBLISHED   acme         ann (acme)    Thu Jan 14 18:32:57 2021\n"
-            "CVE-2021-3003   REJECTED    acme         eve (corp)    Thu Jan 14 18:34:50 2021\n"
+            "CVE-2021-3001   RESERVED    acme         bob (acme)    "
+            "Thu Jan 14 18:32:19 2021 +0000\n"
+            "CVE-2021-3002   PUBLISHED   acme         ann (acme)    "
+            "Thu Jan 14 18:32:57 2021 +0000\n"
+            "CVE-2021-3003   REJECTED    acme         eve (corp)    "
+            "Thu Jan 14 18:34:50 2021 +0000\n"
         )
         result = runner.invoke(cli, DEFAULT_OPTS + ["list", "--no-header"])
         assert result.exit_code == 0, result.output
         assert result.output == (
-            "CVE-2021-3001   RESERVED    acme   bob (acme)   Thu Jan 14 18:32:19 2021\n"
-            "CVE-2021-3002   PUBLISHED   acme   ann (acme)   Thu Jan 14 18:32:57 2021\n"
-            "CVE-2021-3003   REJECTED    acme   eve (corp)   Thu Jan 14 18:34:50 2021\n"
+            "CVE-2021-3001   RESERVED    acme   bob (acme)   Thu Jan 14 18:32:19 2021 +0000\n"
+            "CVE-2021-3002   PUBLISHED   acme   ann (acme)   Thu Jan 14 18:32:57 2021 +0000\n"
+            "CVE-2021-3003   REJECTED    acme   eve (corp)   Thu Jan 14 18:34:50 2021 +0000\n"
         )
 
 
@@ -184,7 +306,8 @@ class TestCvePublish:
             "assignerOrgId": "19f229d4-f3d5-4605-bf93-521fa4499c06",
             "assignerShortName": "test_org",
             "cveId": cve_id,
-            "datePublished": "2001-05-02T00:00:00Z",
+            "datePublished": "2022-05-02T00:00:00Z",
+            "dateUpdated": "2021-06-29T12:33:52.892Z",
             "dateReserved": "2021-06-29T12:33:52.892Z",
             "requesterUserId": "cb55b254-cf8f-46f6-bfbf-fc1d71ba439a",
             "state": "PUBLISHED",
@@ -199,7 +322,7 @@ class TestCvePublish:
         cna_text = json.dumps(self.cna_dict)
         response_dict = {
             "created": self.cve_response_data,
-            "message": f"{self.cve_id} record was " f"successfully created.",
+            "message": f"{self.cve_id} record was successfully created.",
         }
         publish.return_value = response_dict
 
@@ -212,7 +335,9 @@ class TestCvePublish:
             f"{self.cve_id}\n"
             "├─ State:\tPUBLISHED\n"
             "├─ Owning CNA:\ttest_org\n"
-            "└─ Reserved on:\tTue Jun 29 12:33:52 2021\n"
+            "├─ Reserved on:\tTue Jun 29 12:33:52 2021 +0000\n"
+            "└─ Updated on:\tTue Jun 29 12:33:52 2021 +0000\n"
+            f"\nAPI response: {self.cve_id} record was successfully created.\n"
         )
         assert not update_published.called
 
@@ -239,11 +364,50 @@ class TestCvePublish:
             f"{self.cve_id}\n"
             "├─ State:\tPUBLISHED\n"
             "├─ Owning CNA:\ttest_org\n"
-            "└─ Reserved on:\tTue Jun 29 12:33:52 2021\n"
+            "├─ Reserved on:\tTue Jun 29 12:33:52 2021 +0000\n"
+            "└─ Updated on:\tTue Jun 29 12:33:52 2021 +0000\n"
+            f"\nAPI response: {self.cve_id} record was successfully created.\n"
         )
         assert not update_published.called
 
 
+@mock.patch("cvelib.cli.CveApi.publish_adp")
+def test_adp_publish(publish_adp):
+    adp_text = json.dumps({"references": [{"url": "https://example.com/hello"}]})
+    response_dict = {
+        "updated": {
+            "cveMetadata": {
+                "assignerOrgId": "uuid",
+                "assignerShortName": "test_org",
+                "cveId": "CVE-2001-0635",
+                "datePublished": "2022-05-02T00:00:00Z",
+                "dateUpdated": "2021-06-29T12:33:52.892Z",
+                "dateReserved": "2021-06-29T12:33:52.892Z",
+                "state": "PUBLISHED",
+            }
+        },
+        # Omit the rest of the API response message since it may change in the future.
+        "message": "CVE-2001-0635 record had a replacement ADP container...",
+    }
+    publish_adp.return_value = response_dict
+
+    runner = CliRunner()
+    result = runner.invoke(
+        cli, DEFAULT_OPTS + ["publish-adp", "CVE-2001-0635", "--adp-json", adp_text]
+    )
+    assert result.exit_code == 0, result.output
+    assert result.output == (
+        "Published an ADP container for:\n"
+        "\n"
+        "CVE-2001-0635\n"
+        "├─ State:\tPUBLISHED\n"
+        "├─ Owning CNA:\ttest_org\n"
+        "├─ Reserved on:\tTue Jun 29 12:33:52 2021 +0000\n"
+        "└─ Updated on:\tTue Jun 29 12:33:52 2021 +0000\n"
+        "\nAPI response: CVE-2001-0635 record had a replacement ADP container...\n"
+    )
+
+
 @mock.patch("cvelib.cli.CveApi.reject")
 @mock.patch("cvelib.cli.CveApi.update_rejected")
 @mock.patch("cvelib.cli.CveApi.move_to_rejected")
@@ -268,7 +432,8 @@ def test_cve_reject_with_record(move_to_rejected, update_rejected, reject):
             "assignerOrgId": "19f229d4-f3d5-4605-bf93-521fa4499c06",
             "assignerShortName": "test_org",
             "cveId": cve_id,
-            "dateRejected": "2001-05-02T00:00:00Z",
+            "dateRejected": "2022-05-02T00:00:00Z",
+            "dateUpdated": "2022-05-02T00:00:00Z",
             "dateReserved": "2021-06-29T12:33:52.892Z",
             "requesterUserId": "cb55b254-cf8f-46f6-bfbf-fc1d71ba439a",
             "state": "REJECTED",
@@ -289,7 +454,9 @@ def test_cve_reject_with_record(move_to_rejected, update_rejected, reject):
         f"{cve_id}\n"
         "├─ State:\tREJECTED\n"
         "├─ Owning CNA:\ttest_org\n"
-        "└─ Reserved on:\tTue Jun 29 12:33:52 2021\n"
+        "├─ Reserved on:\tTue Jun 29 12:33:52 2021 +0000\n"
+        "└─ Updated on:\tMon May  2 00:00:00 2022 +0000\n"
+        f"\nAPI response: {cve_id} record was successfully created.\n"
     )
     assert not update_rejected.called
     assert not move_to_rejected.called
@@ -324,7 +491,8 @@ def test_cve_reject_without_record(move_to_rejected, update_rejected, reject):
         "├─ State:\tREJECTED\n"
         "├─ Owning CNA:\tacme\n"
         "├─ Reserved by:\tjack@example.com (acme)\n"
-        "└─ Reserved on:\tTue Jun 29 12:33:52 2021\n"
+        "├─ Reserved on:\tTue Jun 29 12:33:52 2021 +0000\n"
+        "└─ Updated on:\tTue Jun 29 12:33:52 2021 +0000\n"
     )
     assert not update_rejected.called
     assert not reject.called
@@ -357,7 +525,8 @@ def test_cve_undo_reject(move_to_reserved):
         "├─ State:\tRESERVED\n"
         "├─ Owning CNA:\tacme\n"
         "├─ Reserved by:\tjack@example.com (acme)\n"
-        "└─ Reserved on:\tTue Jun 29 12:33:52 2021\n"
+        "├─ Reserved on:\tTue Jun 29 12:33:52 2021 +0000\n"
+        "└─ Updated on:\tTue Jun 29 12:33:52 2021 +0000\n"
     )
 
 
@@ -412,12 +581,12 @@ def test_reserve():
             "├─ State:\tRESERVED\n"
             "├─ Owning CNA:\ttest_org\n"
             "├─ Reserved by:\ttest_user@test_org.com (test_org)\n"
-            "└─ Reserved on:\tMon May 24 18:14:34 2021\n"
+            "└─ Reserved on:\tMon May 24 18:14:34 2021 +0000\n"
             "CVE-2021-20002\n"
             "├─ State:\tRESERVED\n"
             "├─ Owning CNA:\ttest_org\n"
             "├─ Reserved by:\ttest_user@test_org.com (test_org)\n"
-            "└─ Reserved on:\tMon May 24 18:14:34 2021\n"
+            "└─ Reserved on:\tMon May 24 18:14:34 2021 +0000\n"
             "\n"
             "Remaining quota: 10\n"
         )
@@ -445,8 +614,8 @@ def test_active_user_show():
             "Test User — test@user\n"
             "├─ Active:\tYes\n"
             "├─ Roles:\tADMIN\n"
-            "├─ Created:\tThu Apr 22 02:09:08 2021\n"
-            "└─ Modified:\tThu Apr 22 02:09:08 2021\n"
+            "├─ Created:\tThu Apr 22 02:09:08 2021 +0000\n"
+            "└─ Modified:\tThu Apr 22 02:09:08 2021 +0000\n"
         )
 
 
@@ -472,8 +641,8 @@ def test_inactive_user_show():
             "test@user\n"
             "├─ Active:\tNo\n"
             "├─ Roles:\tNone\n"
-            "├─ Created:\tThu Apr 22 02:09:08 2021\n"
-            "└─ Modified:\tThu Apr 22 02:09:08 2021\n"
+            "├─ Created:\tThu Apr 22 02:09:08 2021 +0000\n"
+            "└─ Modified:\tThu Apr 22 02:09:08 2021 +0000\n"
         )
 
 
@@ -482,7 +651,7 @@ def test_reset_key():
     with mock.patch("cvelib.cli.CveApi.reset_api_key") as reset_api_key:
         reset_api_key.return_value = api_key
         runner = CliRunner()
-        result = runner.invoke(cli, DEFAULT_OPTS + ["user", "reset-key"])
+        result = runner.invoke(cli, DEFAULT_OPTS + ["user", "reset-key", "-u", "test_user"])
         assert result.exit_code == 0, result.output
         assert result.output == (
             "New API key for test_user:\n\n"
@@ -518,11 +687,12 @@ def test_user_list():
         result = runner.invoke(cli, DEFAULT_OPTS + ["org", "users"])
         assert result.exit_code == 0, result.output
         assert result.output == (
-            "USERNAME      NAME          ROLES   ACTIVE   CREATED                    MODIFIED\n"
-            "foo           Foo           None    No       Wed May 26 19:27:24 2021   "
-            "Wed May 26 19:32:57 2021\n"
-            "hello@world   Hello World   ADMIN   Yes      Wed May 26 19:27:44 2021   "
-            "Wed May 26 19:28:52 2021\n"
+            "USERNAME      NAME          ROLES   ACTIVE   CREATED                          "
+            "MODIFIED\n"
+            "foo           Foo           None    No       Wed May 26 19:27:24 2021 +0000   "
+            "Wed May 26 19:32:57 2021 +0000\n"
+            "hello@world   Hello World   ADMIN   Yes      Wed May 26 19:27:44 2021 +0000   "
+            "Wed May 26 19:28:52 2021 +0000\n"
         )
 
 
@@ -543,6 +713,33 @@ def test_show_org():
         assert result.output == (
             "Test Org — test_org\n"
             "├─ Roles:\tCNA\n"
-            "├─ Created:\tWed Apr 21 02:09:07 2021\n"
-            "└─ Modified:\tWed Apr 21 02:09:07 2021\n"
+            "├─ Created:\tWed Apr 21 02:09:07 2021 +0000\n"
+            "└─ Modified:\tWed Apr 21 02:09:07 2021 +0000\n"
         )
+
+
+def test_validate_good():
+    runner = CliRunner()
+    example_cve_file = str(Path(__file__).parent / "data/CVEv5_basic-example.json")
+    result = runner.invoke(
+        cli, ["validate", "--cve-json-file", example_cve_file, "--schema-type", "full"]
+    )
+    assert result.exit_code == 0, result.output
+    assert result.output == "CVE record is valid!\n"
+
+
+def test_validate_bad():
+    runner = CliRunner()
+    result = runner.invoke(cli, ["validate", "--cve-json", '{"bad": "record"}'])
+    assert result.exit_code == 1
+    assert re.search("^Schema validation .* failed:\n", result.output)
+
+
+def test_exit_on_help():
+    with mock.patch("cvelib.cli.CveApi.show_org") as show_org:
+        show_org.return_value = {}
+        runner = CliRunner()
+        result = runner.invoke(cli, ["org", "--help"])
+        assert result.exit_code == 0, result.output
+        # The command is named after the function that is called.
+        assert result.output.startswith("Usage: cli org")
